-heap 0x0000
-stack 0x0800
/* ECC-Capable memory (Mx, Dx, FLASHx) should make sure to have ALIGN(4) since
 * ECC is calculated for 64-bit data.
 * Secure memory sections: Dx, LSx, CPU/CLA MSGRAM, FLASHx                      */
MEMORY
{
PAGE 0 :    /* Program Memory                                                   */
    RAMM0           : origin = 0x000122, length = 0x0002DE
    RAMLS0          : origin = 0x008000, length = 0x000800
    RAMLS1_2        : origin = 0x008800, length = 0x001000
    RAMD0           : origin = 0x00B000, length = 0x000800

    BEGIN           : origin = 0x080000, length = 0x000002
    FLASHA          : origin = 0x080002, length = 0x001FFE
    FLASHB          : origin = 0x082000, length = 0x002000
    FLASHC          : origin = 0x084000, length = 0x002000
    FLASHD          : origin = 0x086000, length = 0x002000
    FLASHE          : origin = 0x088000, length = 0x008000
    FLASHF          : origin = 0x090000, length = 0x008000
    FLASHG          : origin = 0x098000, length = 0x008000
    FLASHH          : origin = 0x0A0000, length = 0x008000
    FLASHI          : origin = 0x0A8000, length = 0x008000
    FLASHJ          : origin = 0x0B0000, length = 0x008000

    RESET           : origin = 0x3FFFC0, length = 0x000002
PAGE 1 :    /* Data Memory                                                      */
    BOOT_RSVD       : origin = 0x000002, length = 0x000120
    RAMM1           : origin = 0x000400, length = 0x000400
    CLA1_MSGRAMLOW  : origin = 0x001480, length = 0x000080
    CLA1_MSGRAMHIGH : origin = 0x001500, length = 0x000080
    RAMLS3          : origin = 0x009800, length = 0x000800
    RAMLS4_5        : origin = 0x00A000, length = 0x001000
    RAMD1           : origin = 0x00B800, length = 0x000800

    RAMGS0          : origin = 0x00C000, length = 0x001000
    RAMGS1          : origin = 0x00D000, length = 0x001000
    RAMGS2          : origin = 0x00E000, length = 0x001000
    RAMGS3          : origin = 0x00F000, length = 0x001000
    RAMGS4          : origin = 0x010000, length = 0x001000
    RAMGS5          : origin = 0x011000, length = 0x001000
    RAMGS6          : origin = 0x012000, length = 0x001000
    RAMGS7          : origin = 0x013000, length = 0x001000
    RAMGS8          : origin = 0x014000, length = 0x001000
    RAMGS9          : origin = 0x015000, length = 0x001000
    RAMGS10         : origin = 0x016000, length = 0x001000
    RAMGS11         : origin = 0x017000, length = 0x001000
    RAMGS12         : origin = 0x018000, length = 0x001000
    RAMGS13         : origin = 0x019000, length = 0x001000
    RAMGS14         : origin = 0x01A000, length = 0x001000
    RAMGS15         : origin = 0x01B000, length = 0x001000

    FLASHK          : origin = 0x0B8000, length = 0x002000
    FLASHL          : origin = 0x0BA000, length = 0x002000
    FLASHM          : origin = 0x0BC000, length = 0x002000
    FLASHN          : origin = 0x0BE000, length = 0x002000
}


SECTIONS
{
    codestart       : > BEGIN, PAGE = 0, ALIGN(4)
    .cinit          : > FLASHA, PAGE = 0, ALIGN(4)
    .binit          : > FLASHA, PAGE = 0, ALIGN(4)
    .pinit          : > FLASHA, PAGE = 0, ALIGN(4)
    .text           : >> FLASHB | FLASHC | FLASHD, PAGE = 0, ALIGN(4)
#ifdef TABLE_BINIT
   .TI.ramfunc      : LOAD = FLASHB | FLASHC | FLASHD, PAGE = 0, ALIGN(4),
                      RUN = RAMLS0, PAGE = 0,
                      TABLE(BINIT)
    Cla1Prog        : LOAD = FLASHB | FLASHC | FLASHD, PAGE = 0, ALIGN(4),
                      RUN = RAMLS1_2, PAGE = 0,
                      TABLE(BINIT)
#else
   .TI.ramfunc      : LOAD = FLASHB | FLASHC | FLASHD, PAGE = 0, ALIGN(4),
                      RUN = RAMLS0, PAGE = 0,
                      LOAD_START(_RamfuncsLoadStart),
                      LOAD_SIZE(_RamfuncsLoadSize),
                      RUN_START(_RamfuncsRunStart)
    Cla1Prog        : LOAD = FLASHB | FLASHC | FLASHD, PAGE = 0, ALIGN(4),
                      RUN = RAMLS1_2, PAGE = 0,
                      LOAD_START(_Cla1ProgLoadStart),
                      LOAD_SIZE(_Cla1ProgLoadSize),
                      RUN_START(_Cla1ProgRunStart)
#endif

    .econst         : >> FLASHK | FLASHL | FLASHM | FLASHN, PAGE = 1, ALIGN(4)
    .switch         : >> FLASHK | FLASHL | FLASHM | FLASHN, PAGE = 1, ALIGN(4)
    .stack          : > RAMD1, PAGE = 1, ALIGN(4)
    .ebss           : > RAMLS3, PAGE = 1
    Cla1ToCpuMsgRAM : > CLA1_MSGRAMLOW, PAGE = 1
    CpuToCla1MsgRAM : > CLA1_MSGRAMHIGH, PAGE = 1
    .scratchpad     : > RAMLS4_5, PAGE = 1
    .bss_cla        : > RAMLS4_5, PAGE = 1
    .const_cla      : > RAMLS4_5, PAGE = 1

#ifdef TABLE_BINIT
    CLA1mathTables  : LOAD = FLASHB | FLASHC | FLASHD, PAGE = 0, ALIGN(4),
                      RUN = RAMLS4_5, PAGE = 1,
                      TABLE(BINIT)
#else
    CLA1mathTables  : LOAD = FLASHB | FLASHC | FLASHD, PAGE = 0, ALIGN(4),
                      RUN = RAMLS4_5, PAGE = 1,
                      LOAD_START(_CLA1mathTablesLoadStart),
                      LOAD_SIZE(_CLA1mathTablesLoadSize),
                      RUN_START(_CLA1mathTablesRunStart)
#endif

    .reset          : > RESET, PAGE = 0, TYPE = DSECT
}
