 #include "test.h"

#define TMR1SYSCLKCTR        0xF0000000
#define TMR2INPCLKCTR        0x800

#pragma CODE_SECTION(InitFlash, ".TI.ramfunc");

static void InitFlash(void);
static void InitPeripheralClocks(void);
static void InitSysPll(void);

void InitSysCtrl(void) {
    Uint16 pin_count;

    DisableDog();

#ifndef TABLE_BINIT
    memcpy(&RamfuncsRunStart, &RamfuncsLoadStart, (Uint32)&RamfuncsLoadSize);
    memcpy(&Cla1ProgRunStart, &Cla1ProgLoadStart, (Uint32)&Cla1ProgLoadSize);
    memcpy(&CLA1mathTablesRunStart, &CLA1mathTablesLoadStart, (Uint32)&CLA1mathTablesLoadSize);
#endif

    InitFlash();

    EALLOW;
    /* Enable unbonded IO pullups
     * 5 = 100 pin
     * 6 = 176 pin
     * 7 = 337 pin                                                              */
    pin_count = DevCfgRegs.PARTIDL.bit.PIN_COUNT;
    if (pin_count == 5) {                       /* GPIO #                       */
        GpioCtrlRegs.GPAPUD.all = ~0xFFC003E3;  /* 0-1, 5-9, 22-31              */
        GpioCtrlRegs.GPBPUD.all = ~0x03FFF1FF;  /* 32-40, 44-57                 */
        GpioCtrlRegs.GPCPUD.all = ~0xE10FBC18;  /* 67-68, 74-77, 79-83, 93-95   */
        GpioCtrlRegs.GPDPUD.all = ~0xFFFFFFF7;  /* 96-127                       */
        GpioCtrlRegs.GPEPUD.all = ~0xFFFFFFFF;  /* 128-159                      */
        GpioCtrlRegs.GPFPUD.all = ~0x000001FF;  /* 160-168                      */
    }
    else if (pin_count == 6) {
        GpioCtrlRegs.GPCPUD.all = ~0x80000000;  /* 95                           */
        GpioCtrlRegs.GPDPUD.all = ~0xFFFFFFF7;  /* 96-127                       */
        GpioCtrlRegs.GPEPUD.all = ~0xFFFFFFDF;  /* 128-159 except for 133       */
        GpioCtrlRegs.GPFPUD.all = ~0x000001FF;  /* 160-168                      */
    }

    CpuSysRegs.PCLKCR13.bit.ADC_A = 1;
    CpuSysRegs.PCLKCR13.bit.ADC_B = 1;
    CpuSysRegs.PCLKCR13.bit.ADC_C = 1;
    CpuSysRegs.PCLKCR13.bit.ADC_D = 1;

    /* Check if device is trimmed                                               */
    if (*((Uint16 *)0x5D1B6) == 0x0000) {
        /* Device is not trimmed--apply static calibration values               */
        AnalogSubsysRegs.ANAREFTRIMA.all = 31709;
        AnalogSubsysRegs.ANAREFTRIMB.all = 31709;
        AnalogSubsysRegs.ANAREFTRIMC.all = 31709;
        AnalogSubsysRegs.ANAREFTRIMD.all = 31709;
    }

    CpuSysRegs.PCLKCR13.bit.ADC_A = 0;
    CpuSysRegs.PCLKCR13.bit.ADC_B = 0;
    CpuSysRegs.PCLKCR13.bit.ADC_C = 0;
    CpuSysRegs.PCLKCR13.bit.ADC_D = 0;
    EDIS;

    InitSysPll();

    InitPeripheralClocks();
}

static void InitPeripheralClocks(void) {
    EALLOW;

    CpuSysRegs.PCLKCR0.bit.CLA1 = 1;
    CpuSysRegs.PCLKCR0.bit.DMA = 1;
    CpuSysRegs.PCLKCR0.bit.CPUTIMER0 = 1;
    CpuSysRegs.PCLKCR0.bit.CPUTIMER1 = 1;
    CpuSysRegs.PCLKCR0.bit.CPUTIMER2 = 1;
    CpuSysRegs.PCLKCR0.bit.HRPWM = 1;
    CpuSysRegs.PCLKCR0.bit.TBCLKSYNC = 1;

    CpuSysRegs.PCLKCR1.bit.EMIF1 = 1;
    CpuSysRegs.PCLKCR1.bit.EMIF2 = 1;

    CpuSysRegs.PCLKCR2.bit.EPWM1 = 1;
    CpuSysRegs.PCLKCR2.bit.EPWM2 = 1;
    CpuSysRegs.PCLKCR2.bit.EPWM3 = 1;
    CpuSysRegs.PCLKCR2.bit.EPWM4 = 1;
    CpuSysRegs.PCLKCR2.bit.EPWM5 = 1;
    CpuSysRegs.PCLKCR2.bit.EPWM6 = 1;
    CpuSysRegs.PCLKCR2.bit.EPWM7 = 1;
    CpuSysRegs.PCLKCR2.bit.EPWM8 = 1;
    CpuSysRegs.PCLKCR2.bit.EPWM9 = 1;
    CpuSysRegs.PCLKCR2.bit.EPWM10 = 1;
    CpuSysRegs.PCLKCR2.bit.EPWM11 = 1;
    CpuSysRegs.PCLKCR2.bit.EPWM12 = 1;

    CpuSysRegs.PCLKCR3.bit.ECAP1 = 1;
    CpuSysRegs.PCLKCR3.bit.ECAP2 = 1;
    CpuSysRegs.PCLKCR3.bit.ECAP3 = 1;
    CpuSysRegs.PCLKCR3.bit.ECAP4 = 1;
    CpuSysRegs.PCLKCR3.bit.ECAP5 = 1;
    CpuSysRegs.PCLKCR3.bit.ECAP6 = 1;

    CpuSysRegs.PCLKCR4.bit.EQEP1 = 1;
    CpuSysRegs.PCLKCR4.bit.EQEP2 = 1;
    CpuSysRegs.PCLKCR4.bit.EQEP3 = 1;

    CpuSysRegs.PCLKCR6.bit.SD1 = 1;
    CpuSysRegs.PCLKCR6.bit.SD2 = 1;

    CpuSysRegs.PCLKCR7.bit.SCI_A = 1;
    CpuSysRegs.PCLKCR7.bit.SCI_B = 1;
    CpuSysRegs.PCLKCR7.bit.SCI_C = 1;
    CpuSysRegs.PCLKCR7.bit.SCI_D = 1;

    CpuSysRegs.PCLKCR8.bit.SPI_A = 1;
    CpuSysRegs.PCLKCR8.bit.SPI_B = 1;
    CpuSysRegs.PCLKCR8.bit.SPI_C = 1;

    CpuSysRegs.PCLKCR9.bit.I2C_A = 1;
    CpuSysRegs.PCLKCR9.bit.I2C_B = 1;

    CpuSysRegs.PCLKCR10.bit.CAN_A = 1;
    CpuSysRegs.PCLKCR10.bit.CAN_B = 1;

    CpuSysRegs.PCLKCR11.bit.McBSP_A = 1;
    CpuSysRegs.PCLKCR11.bit.McBSP_B = 1;
    CpuSysRegs.PCLKCR11.bit.USB_A = 1;

    CpuSysRegs.PCLKCR12.bit.uPP_A = 1;

    CpuSysRegs.PCLKCR13.bit.ADC_A = 1;
    CpuSysRegs.PCLKCR13.bit.ADC_B = 1;
    CpuSysRegs.PCLKCR13.bit.ADC_C = 1;
    CpuSysRegs.PCLKCR13.bit.ADC_D = 1;

    CpuSysRegs.PCLKCR14.bit.CMPSS1 = 1;
    CpuSysRegs.PCLKCR14.bit.CMPSS2 = 1;
    CpuSysRegs.PCLKCR14.bit.CMPSS3 = 1;
    CpuSysRegs.PCLKCR14.bit.CMPSS4 = 1;
    CpuSysRegs.PCLKCR14.bit.CMPSS5 = 1;
    CpuSysRegs.PCLKCR14.bit.CMPSS6 = 1;
    CpuSysRegs.PCLKCR14.bit.CMPSS7 = 1;
    CpuSysRegs.PCLKCR14.bit.CMPSS8 = 1;

    CpuSysRegs.PCLKCR16.bit.DAC_A = 1;
    CpuSysRegs.PCLKCR16.bit.DAC_B = 1;
    CpuSysRegs.PCLKCR16.bit.DAC_C = 1;

    EDIS;
}

static void InitFlash(void) {
    EALLOW;

    /* The default value of VREADST is good enough for the flash to power up
     * properly at the INTOSC frequency. Below VREADST configuration covers up
     * to the max frequency possible for this device. This is required for
     * proper flash wake up at the higher frequencies if users put it to sleep
     * for power saving reason.                                                 */
    Flash0CtrlRegs.FBAC.bit.VREADST = 0x14;

    /* At reset bank and pump are in sleep. A Flash access will power up the
     * bank and pump automatically.
     * After a Flash access, bank and pump go to low power mode (configurable
     * in FBFALLBACK/FPAC1 registers) if there is no further access to flash.
     * Power up Flash bank and pump. This also sets the fall back mode of
     * flash and pump as active.                                                */
    Flash0CtrlRegs.FPAC1.bit.PMPPWR = 0x1;
    Flash0CtrlRegs.FBFALLBACK.bit.BNKPWR0 = 0x3;

    /* Disable Cache and prefetch mechanism before changing wait states         */
    Flash0CtrlRegs.FRD_INTF_CTRL.bit.DATA_CACHE_EN = 0;
    Flash0CtrlRegs.FRD_INTF_CTRL.bit.PREFETCH_EN = 0;

    /* Set waitstates according to frequency
     * Minimum waitstates required for the flash operating at a given CPU rate
     * must be characterized by TI. Refer to the datasheet for the latest
     * information.                                                             */
    Flash0CtrlRegs.FRDCNTL.bit.RWAIT = 0x3;

    /* Enable Cache and prefetch mechanism to improve performance of code
     * executed from Flash.                                                     */
    Flash0CtrlRegs.FRD_INTF_CTRL.bit.DATA_CACHE_EN = 1;
    Flash0CtrlRegs.FRD_INTF_CTRL.bit.PREFETCH_EN = 1;

    /* At reset, ECC is enabled. If it is disabled by application software and
     * if application again wants to enable ECC.                                */
    Flash0EccRegs.ECC_ENABLE.bit.ENABLE = 0xA;

    EDIS;

    /* Force a pipeline flush to ensure that the write to the last register
     * configured occurs before returning.                                      */
    __asm(" RPT #7 || NOP");
}

void ServiceDog(void) {
    EALLOW;
    WdRegs.WDKEY.bit.WDKEY = 0x0055;
    WdRegs.WDKEY.bit.WDKEY = 0x00AA;
    EDIS;
}

void DisableDog(void) {
    volatile Uint16 temp;
    /* Grab the clock config first so we don't clobber it                       */
    EALLOW;
    temp = WdRegs.WDCR.all & 0x0007;
    WdRegs.WDCR.all = temp | 0x0068;
    EDIS;
}

/* InitSysPll()
 * This function initializes the PLL registers.
 * Note:
 *  - The internal oscillator CANNOT be used as the PLL source if the
 *    PLLSYSCLK is configured to frequencies above 194 MHz.
 *
 *  - This function uses the Watchdog as a monitor for the PLL. The user
 *  watchdog settings will be modified and restored upon completion.  Function
 *  allows for a minimum re lock attempt for 5 tries.  Re lock attempt is carried
 *  out if either SLIP condition occurs or SYSCLK to Input Clock ratio is off by 10%
 *
 *  - This function uses the following resources to support PLL initialization:
 *          o Watchdog
 *          o CPU Timer 1
 *          o CPU Timer 2                                                       */
static void InitSysPll(void) {
    Uint16 SCSRtemp, WDCRtemp, WDWCRtemp, intStatus, t1TCR, t1TPR, t1TPRH;
    Uint16 t2TCR, t2TPR, t2TPRH, t2SRC, t2Prescale;
    Uint16 i, a, b;
    Uint32 t1PRD, t2PRD, ctr1;
    float32 sysclkToInClkError, mult, divi;
    bool sysclkInvalidFreq = true;

    EALLOW;
    ClkCfgRegs.CLKSRCCTL1.bit.XTALOFF = 0;
    ClkCfgRegs.CLKSRCCTL1.bit.OSCCLKSRCSEL = 1;

    if (DevCfgRegs.SYSDBGCTL.bit.BIT_0 == 1) {
        /* This will only be executed if a watchdog reset occurred after a
         * failed system PLL initialization.                                    */
    }

    /* Bypass PLL and set dividers to /1                                        */
    ClkCfgRegs.SYSPLLCTL1.bit.PLLCLKEN = 0;
    __asm(" RPT #20 || NOP");
    ClkCfgRegs.SYSCLKDIVSEL.bit.PLLSYSCLKDIV = 0;

    /* Lock the PLL 5 times                                                     */
    for (i = 0; i < 5; i++) {
        /* Turn off PLL */
        ClkCfgRegs.SYSPLLCTL1.bit.PLLEN = 0;
        __asm(" RPT #20 || NOP");
        /* Write multiplier, which automatically turns on the PLL               */
        ClkCfgRegs.SYSPLLMULT.all = 20;
        /* Wait for the SYSPLL lock counter                                     */
        while (ClkCfgRegs.SYSPLLSTS.bit.LOCKS != 1);
    }

    /* Set divider to produce slower output frequency to limit current increase */
    ClkCfgRegs.SYSCLKDIVSEL.bit.PLLSYSCLKDIV = 2;

    /* Backup User Watchdog                                                     */
    SCSRtemp = WdRegs.SCSR.all;
    WDCRtemp = WdRegs.WDCR.all;
    WDWCRtemp = WdRegs.WDWCR.all;

    /* Disable windowed functionality, reset counter                            */
    WdRegs.WDWCR.all = 0x0;
    WdRegs.WDKEY.bit.WDKEY = 0x55;
    WdRegs.WDKEY.bit.WDKEY = 0xAA;

    /* Disable global interrupts                                                */
    intStatus = __disable_interrupts();

    /* Configure for watchdog reset and to run at max frequency                 */
    WdRegs.SCSR.all = 0x0;
    WdRegs.WDCR.all = 0x28;

    /* This bit is reset only by power-on-reset (POR) and will not be cleared
     * by a WD reset                                                            */
    DevCfgRegs.SYSDBGCTL.bit.BIT_0 = 1;

    /* Enable PLLSYSCLK is fed from system PLL clock                            */
    ClkCfgRegs.SYSPLLCTL1.bit.PLLCLKEN = 1;

    /* Delay to ensure system is clocking from PLL prior to clearing status bit */
    __asm(" RPT #20 || NOP");
    EDIS;

    /* Service watchdog                                                         */
    ServiceDog();

    EALLOW;
    while (sysclkInvalidFreq == true) {
        if (ClkCfgRegs.SYSPLLSTS.bit.SLIPS == 1) {
            /* Bypass PLL                                                       */
            ClkCfgRegs.SYSPLLCTL1.bit.PLLCLKEN = 0;
            __asm(" RPT #20 || NOP");
            /* Turn off PLL                                                     */
            ClkCfgRegs.SYSPLLCTL1.bit.PLLEN = 0;
            __asm(" RPT #20 || NOP");
            /* Write multipliers, which automatically turns on the PLL          */
            ClkCfgRegs.SYSPLLMULT.all = 20;
            /* Wait for the SYSPLL lock counter to expire                       */
            while (ClkCfgRegs.SYSPLLSTS.bit.LOCKS != 1);
            /* Enable PLLSYSCLK being fed from system PLL clock                 */
            ClkCfgRegs.SYSPLLCTL1.bit.PLLCLKEN = 1;
            /* Delay to ensure system is clocking from PLL                      */
            __asm(" RPT #20 || NOP");
        }
        /* Backup timer1 and timer2 settings                                    */
        t1TCR = CpuTimer1Regs.TCR.all;
        t1PRD = CpuTimer1Regs.PRD.all;
        t1TPR = CpuTimer1Regs.TPR.all;
        t1TPRH = CpuTimer1Regs.TPRH.all;
        t2SRC = CpuSysRegs.TMR2CLKCTL.bit.TMR2CLKSRCSEL;
        t2Prescale = CpuSysRegs.TMR2CLKCTL.bit.TMR2CLKPRESCALE;
        t2TCR = CpuTimer2Regs.TCR.all;
        t2PRD = CpuTimer2Regs.PRD.all;
        t2TPR = CpuTimer2Regs.TPR.all;
        t2TPRH = CpuTimer2Regs.TPRH.all;

        /* Set up timers 1 and 2                                                */
        /* Configure timer1 to count SYSCLK cycles                              */
        CpuTimer1Regs.TCR.bit.TSS = 1;
        CpuTimer1Regs.PRD.all = TMR1SYSCLKCTR;
        CpuTimer1Regs.TPR.bit.TDDR = 0x0;
        CpuTimer1Regs.TCR.bit.TRB = 1;
        CpuTimer1Regs.TCR.bit.TIF = 1;
        CpuTimer1Regs.TCR.bit.TIE = 1;

        /* Configure timer2 to count Input clock cycles                         */
        CpuSysRegs.TMR2CLKCTL.bit.TMR2CLKSRCSEL = 0x3;
        CpuTimer2Regs.TCR.bit.TIF = 1;
        CpuTimer2Regs.TCR.bit.TIE = 1;
        CpuTimer2Regs.TCR.bit.TSS = 1;
        CpuTimer2Regs.PRD.all = TMR2INPCLKCTR;
        CpuTimer2Regs.TPR.bit.TDDR = 0x0;
        CpuTimer2Regs.TCR.bit.TRB = 1;

        /* Stop/Start timer counters                                            */
        CpuTimer1Regs.TCR.bit.TSS = 1;
        CpuTimer2Regs.TCR.bit.TSS = 1;
        CpuTimer1Regs.TCR.bit.TRB = 1;
        CpuTimer2Regs.TCR.bit.TRB = 1;
        CpuTimer2Regs.TCR.bit.TIF = 1;
        CpuTimer2Regs.TCR.bit.TSS = 0;
        CpuTimer1Regs.TCR.bit.TSS = 0;

        /* Stop timers if either timer1 or timer2 counters overflow             */
        a = CpuTimer1Regs.TCR.bit.TIF;
        b = CpuTimer2Regs.TCR.bit.TIF;
        while ((a == 0) && (b == 0)) {
            a = CpuTimer1Regs.TCR.bit.TIF;
            b = CpuTimer2Regs.TCR.bit.TIF;
        }
        CpuTimer1Regs.TCR.bit.TSS = 1;
        CpuTimer2Regs.TCR.bit.TSS = 1;

        /* Calculate elapsed counts on timer1                                   */
        ctr1 = TMR1SYSCLKCTR - CpuTimer1Regs.TIM.all;
        /* Restore timer settings                                               */
        CpuTimer1Regs.TCR.all = t1TCR;
        CpuTimer1Regs.PRD.all = t1PRD;
        CpuTimer1Regs.TPR.all = t1TPR;
        CpuTimer1Regs.TPRH.all = t1TPRH;
        CpuSysRegs.TMR2CLKCTL.bit.TMR2CLKSRCSEL = t2SRC;
        CpuSysRegs.TMR2CLKCTL.bit.TMR2CLKPRESCALE = t2Prescale;
        CpuTimer2Regs.TCR.all = t2TCR;
        CpuTimer2Regs.PRD.all = t2PRD;
        CpuTimer2Regs.TPR.all = t2TPR;
        CpuTimer2Regs.TPRH.all = t2TPRH;

        /* Calculate Clock Error:
         * Error = (mult/divi) - (timer1 count/timer2 count)                     */
        mult = (float32)20;
        divi = (float32)((ClkCfgRegs.SYSCLKDIVSEL.bit.PLLSYSCLKDIV == 0) ? 1 : (ClkCfgRegs.SYSCLKDIVSEL.bit.PLLSYSCLKDIV << 1));
        sysclkToInClkError = (mult / divi) - ((float32)ctr1 / (float32)TMR2INPCLKCTR);
        sysclkInvalidFreq = ((sysclkToInClkError > 0.10) || (sysclkToInClkError < -0.10));
    }

    /* Clear bit                                                                */
    DevCfgRegs.SYSDBGCTL.bit.BIT_0 = 0;

    /* Restore user watchdog, first resetting counter                           */
    WdRegs.WDKEY.bit.WDKEY = 0x55;
    WdRegs.WDKEY.bit.WDKEY = 0xAA;

    WDCRtemp |= 0x28;                 /* Setup WD key--KEY bits always read 0   */
    WdRegs.WDCR.all = WDCRtemp;
    WdRegs.WDWCR.all = WDWCRtemp;
    WdRegs.SCSR.all = SCSRtemp & 0xFFFE;  /* Mask write to bit 0 (W1toClr)      */

    /* Restore state of ST1[INTM]. This was set by the __disable_interrupts()
     * intrinsic previously.                                                    */
    if ((intStatus & 0x1) == 0) EINT;
    /* Restore state of ST1[DBGM]. This was set by the __disable_interrupts()
     * intrinsic previously.                                                    */
    if ((intStatus & 0x2) == 0) ERTM;
    /* 200 PLLSYSCLK delay to allow voltage regulator to stabilize prior
     * to increasing entire system clock frequency.                             */
    __asm(" RPT #200 || NOP");
    /* Set the divider to divide by 2                                           */
    ClkCfgRegs.SYSCLKDIVSEL.bit.PLLSYSCLKDIV = 1;

    EDIS;
}
